@@ -89,42 +89,42 @@
 #define TK_OFFSET                         89
 #define TK_ON                             90
 #define TK_OR                             91
-#define TK_ORACLE_OUTER_JOIN              92
-#define TK_ORDER                          93
-#define TK_PLUS                           94
-#define TK_PRAGMA                         95
-#define TK_PRIMARY                        96
-#define TK_RAISE                          97
-#define TK_REFERENCES                     98
-#define TK_REM                            99
-#define TK_REPLACE                        100
-#define TK_RESTRICT                       101
-#define TK_ROLLBACK                       102
-#define TK_ROW                            103
-#define TK_RP                             104
-#define TK_RSHIFT                         105
-#define TK_SELECT                         106
-#define TK_SEMI                           107
-#define TK_SET                            108
-#define TK_SLASH                          109
-#define TK_SPACE                          110
-#define TK_STAR                           111
-#define TK_STATEMENT                      112
-#define TK_STRING                         113
-#define TK_TABLE                          114
-#define TK_TEMP                           115
-#define TK_THEN                           116
-#define TK_TRANSACTION                    117
-#define TK_TRIGGER                        118
-#define TK_UMINUS                         119
-#define TK_UNCLOSED_STRING                120
-#define TK_UNION                          121
-#define TK_UNIQUE                         122
-#define TK_UPDATE                         123
-#define TK_UPLUS                          124
-#define TK_USING                          125
-#define TK_VACUUM                         126
-#define TK_VALUES                         127
+#define TK_ORDER                          92
+#define TK_PLUS                           93
+#define TK_PRAGMA                         94
+#define TK_PRIMARY                        95
+#define TK_RAISE                          96
+#define TK_REFERENCES                     97
+#define TK_REM                            98
+#define TK_REPLACE                        99
+#define TK_RESTRICT                       100
+#define TK_ROLLBACK                       101
+#define TK_ROW                            102
+#define TK_RP                             103
+#define TK_RSHIFT                         104
+#define TK_SELECT                         105
+#define TK_SEMI                           106
+#define TK_SET                            107
+#define TK_SLASH                          108
+#define TK_SPACE                          109
+#define TK_STAR                           110
+#define TK_STATEMENT                      111
+#define TK_STRING                         112
+#define TK_TABLE                          113
+#define TK_TEMP                           114
+#define TK_THEN                           115
+#define TK_TRANSACTION                    116
+#define TK_TRIGGER                        117
+#define TK_UMINUS                         118
+#define TK_UNCLOSED_STRING                119
+#define TK_UNION                          120
+#define TK_UNIQUE                         121
+#define TK_UPDATE                         122
+#define TK_UPLUS                          123
+#define TK_USING                          124
+#define TK_VACUUM                         125
+#define TK_VALUES                         126
+#define TK_VARIABLE                       127
 #define TK_VIEW                           128
 #define TK_WHEN                           129
 #define TK_WHERE                          130
